﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.Serialization;

namespace HMS
{
    /// <summary>
    /// Třída obsahující nastavení pro View.
    /// </summary>
    public class GuiSettings
    {
        private const string Path = "GUI_settings.xml";

        public const string Czech = "cs";
        public const string English = "en";

        /// <summary>
        /// String určující lokalizaci.
        /// </summary>
        public string Localization = string.Empty;

        /// <summary>
        /// Načte nastavení z disku.
        /// </summary>
        public void GetSettings()
        {
            try
            {
                var serializer = new XmlSerializer(typeof(GuiSettings));
                using (var reader = XmlReader.Create(Path))
                {
                    var settings = (GuiSettings)serializer.Deserialize(reader);

                    Localization = settings.Localization;
                }
            }
            catch
            {
                Localization = Czech;
            }
        }

        /// <summary>
        /// Uloží aktuální nastavení na disk.
        /// </summary>
        public void SaveSettings()
        {
            var serializer = new XmlSerializer(this.GetType());
            using (var writer = XmlWriter.Create(Path))
            {
                serializer.Serialize(writer, this);
            }
        }
    }
}
