﻿using System;
using HMS.Communication;
using System.Diagnostics;
using System.Windows.Interop;

namespace Model.Connection
{
    /// <summary>
    /// Třída poskytující metody pro připojení k jednotce a odpojení od jednotky
    /// </summary>
    static class Connection
    {
        /// <summary>
        /// Aktuální připojení
        /// </summary>
        private static IComm _communication { get; set; }

        /// <summary>
        /// Metoda sloužící pro připojení k jednotce pomocí TCP.
        /// </summary>
        /// <param name="ipAddress">IP adresa jednotky</param>
        /// <param name="port">Port jednotky</param>
        /// <returns>Připojení</returns>
        public static IComm Connect(string ipAddress, int port)
        {
            try
            {
                Disconnect();
                _communication = null;
                _communication = new TcpCommDevice(ipAddress, port);

                _communication.OpenPort();
                return _communication;
            }
            catch (System.Net.Sockets.SocketException e)
            {
                Debug.WriteLine(e.Message);
                return null;
            }
        }

        /// <summary>
        /// Metoda sloužící pro připojení k jednotce pomocí HID.
        /// </summary>
        /// <param name="source">HwndSource okna.</param>
        /// <returns>Připojení</returns>
        public static IComm Connect(HwndSource source)
        {
            try
            {
                Disconnect();
                _communication = null;
                _communication = new HidCommDevice(source.Handle);

                if (!_communication.OpenPort())
                {
                    Disconnect();
                    return null;
                }

                return _communication;
            }
            catch (Exception e)
            {
                Debug.WriteLine(e.Message);
                return null;
            }
        }

        /// <summary>
        /// Zajišťuje odpojení od jednotky.
        /// </summary>
        public static void Disconnect()
        {
            if (_communication == null) return;
            try
            {
                _communication.CloseCommunications();
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }
        }
    }
}
