﻿using HMS.Communication;
using System.Windows.Interop;

namespace Model
{
    /// <summary>
    /// Třída obsahující základní informace o místnosti.
    /// </summary>
    public class InfoRoom
    {
        public int Id { get; set; }
        public string Title { get; set; }
        public float OnlineTemperature { get; set; }
        public float RequiredTemperature { get; set; }
        public bool IsHeating { get; set; }
        public bool IsWindowOpen { get; set; }
        public bool IsCardActive { get; set; }

        /// <summary>
        /// Vytvoření třídy pro danou místnost.
        /// </summary>
        /// <param name="index"></param>
        public InfoRoom(int index)
        {
            Id = index;
        }

        /// <summary>
        /// Načtení dat pomocí HID.
        /// </summary>
        /// <param name="source">HwndSource</param>
        /// <returns>Pokud bylo načtení úspěšné, vrátí se true, jinak false.</returns>
        public bool Refresh(HwndSource source)
        {
            return Fill(Connection.Connection.Connect(source));
        }

        /// <summary>
        /// Načtení dat pomocí TCP.
        /// </summary>
        /// <param name="ipAddress">IP adresa</param>
        /// <param name="port">Port</param>
        /// <returns>Pokud bylo načtení úspěšné, vrátí se true, jinak false.</returns>
        public bool Refresh(string ipAddress, int port)
        {
            return Fill(Connection.Connection.Connect(ipAddress, port));
        }

        private bool Fill(IComm communication)
        {
            if (communication == null)
            {
                return false;
            }

            Title = communication.GetHeatCircleTitle(Id);
            OnlineTemperature = communication.GetOnlineHeatCircleTemperature(Id);
            RequiredTemperature = communication.GetOnlineHeatCircleRequiredTemperature(Id);
            IsHeating = communication.GetOnlineHeatCircleHeating(Id);
            IsWindowOpen = communication.GetOnlineHeatCircleWindow(Id);
            IsCardActive = communication.GetOnlineHeatCircleCard(Id);

            return true;
        }
    }
}
