﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Input;
using System.Windows.Interop;

namespace ViewModel
{
    public delegate void ConnectionErrorHandler();

    public class InfoRoomList : BaseViewModel
    {
        public event ConnectionErrorHandler OnConnectionError;

        private DateTime _lastRefresh;

        public DateTime LastRefresh
        {
            get { return _lastRefresh; }
            set
            {
                _lastRefresh = value;
                OnPropertyChangedEvent(nameof(LastRefresh));
            }
        }

        private DateTime _lastRefreshError;

        public DateTime LastRefreshError
        {
            get { return _lastRefreshError; }
            set
            {
                _lastRefreshError = value;
                OnPropertyChangedEvent(nameof(LastRefreshError));
            }
        }

        private bool _infoRoomUpdating;

        public bool InfoRoomUpdating
        {
            get { return _infoRoomUpdating; }
            set
            {
                _infoRoomUpdating = value;
                OnPropertyChangedEvent(nameof(InfoRoomUpdating));
            }
        }

        private readonly ObservableCollection<Model.InfoRoom> _infoList = new ObservableCollection<Model.InfoRoom>();

        public IEnumerable<Model.InfoRoom> InfoList
        {
            get { return _infoList; }
        }

        private RelayCommand _refresh;

        public ICommand Refresh
        {
            get
            {
                return _refresh ?? (_refresh = new RelayCommand(async p => await RefreshInfoRoomAsync()));
            }
        }

        public void RefreshInfoRoom()
        {
            try
            {
                InfoRoomUpdating = true;
                if (GlobalInformations.UsingTcp)
                {
                    if (string.IsNullOrEmpty(GlobalInformations.IpAddress))
                    {
                        LastRefreshError = DateTime.Now;
                        return;
                    }

                    RefreshInfoRoom(GlobalInformations.IpAddress, GlobalInformations.Port);
                }
                else
                {
                    RefreshInfoRoom(GlobalInformations.Source);
                }
                InfoRoomUpdating = false;
            }
            catch
            {
                LastRefreshError = DateTime.Now;
                InfoRoomUpdating = false;
                OnConnectionError?.Invoke();
            }
        }

        public Task RefreshInfoRoomAsync()
        {
            return Task.Factory.StartNew(() => RefreshInfoRoom());
        }

        public void RefreshInfoRoom(HwndSource source)
        {
            CheckNumberOfCircles(source);

            foreach (var info in _infoList)
            {
                if (!info.Refresh(source))
                {
                    LastRefreshError = DateTime.Now;
                    return;
                }
            }

            LastRefresh = DateTime.Now;
        }

        public void RefreshInfoRoom(string ipAddress, int port)
        {
            CheckNumberOfCircles(ipAddress, port);

            foreach(var info in _infoList)
            {
                if (!info.Refresh(ipAddress, port))
                {
                    LastRefreshError = DateTime.Now;
                    return;
                }
            }

            LastRefresh = DateTime.Now;
        }

        private void CheckNumberOfCircles(HwndSource source)
        {
            if (_infoList.Any())
            {
                return;
            }

            var settings = new Model.UnitSettings();
            settings.Refresh(source);

            for (var i = 0; i < settings.NumberOfCircles; i++)
            {
                _infoList.Add(new Model.InfoRoom(i));
            }
        }

        private void CheckNumberOfCircles(string ipAddress, int port)
        {
            if (_infoList.Any())
            {
                return;
            }

            var settings = new Model.UnitSettings();
            settings.Refresh(ipAddress, port);

            for (var i = 0; i < settings.NumberOfCircles; i++)
            {
                _infoList.Add(new Model.InfoRoom(i));
            }
        }
    }
}
