﻿using System.Net;
using System.Threading.Tasks;
using System.Windows.Input;

namespace ViewModel
{
    public delegate void TcpConnectionHandler();

    public class IpAddressSelection : BaseViewModel
    {
        public event TcpConnectionHandler OnTcpConnection;

        private string _ipAddress;

        /// <summary>
        /// Obsahuje zadanou IP adresu ve formě stringu.
        /// </summary>
        public string IpAddress
        {
            get { return _ipAddress; }
            set
            {
                _ipAddress = value;
                OnPropertyChangedEvent(nameof(IpAddress));
            }
        }

        private int _port = 502;

        /// <summary>
        /// Obsahuje zadaný port.
        /// </summary>
        public int Port
        {
            get { return _port; }
            set
            {
                _port = value;
                OnPropertyChangedEvent(nameof(Port));
            }
        }

        private RelayCommand _tcpConnection;

        /// <summary>
        /// Příkaz, který provede připojení k jednotce.
        /// </summary>
        public ICommand TcpConnection
        {
            get
            {
                IPAddress ipAddress;
                return _tcpConnection ??
                    (_tcpConnection = new RelayCommand(async p => await TcpConnectionAsync(), 
                    p => IPAddress.TryParse(IpAddress, out ipAddress)));
            }
        }

        public IpAddressSelection()
        {
            if (!string.IsNullOrEmpty(GlobalInformations.IpAddress))
            {
                IpAddress = GlobalInformations.IpAddress;
                Port = GlobalInformations.Port;
            }
        }

        private void TcpConnectionMethod()
        {
            GlobalInformations.UsingTcp = true;
            GlobalInformations.IpAddress = IpAddress;
            GlobalInformations.Port = Port;
            OnTcpConnection?.Invoke();
        }

        private Task TcpConnectionAsync()
        {
            return Task.Factory.StartNew(() => TcpConnectionMethod());
        }
    }
}
