﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Input;

namespace ViewModel
{
    public class MainWindow : BaseViewModel
    {
        private BaseViewModel _menuViewModel;

        /// <summary>
        /// Obsahuje ViewModel s menu.
        /// </summary>
        public BaseViewModel MenuViewModel
        {
            get { return _menuViewModel; }
            set
            {
                _menuViewModel = value;
                OnPropertyChangedEvent(nameof(MenuViewModel));
            }
        }

        private BaseViewModel _fullScreenViewModel;

        /// <summary>
        /// Obsahuje ViewModel pro celoobrazovkové zobrazení (například přihlášení).
        /// </summary>
        public BaseViewModel FullScreenViewModel
        {
            get { return _fullScreenViewModel; }
            set
            {
                _fullScreenViewModel = value;
                OnPropertyChangedEvent(nameof(FullScreenViewModel));
            }
        }

        private BaseViewModel _pageViewModel;

        /// <summary>
        /// Obsahuje ViewModel pro konkrétní zobrazovanou stránku.
        /// </summary>
        public BaseViewModel PageViewModel
        {
            get { return _pageViewModel; }
            set
            {
                _pageViewModel = value;
                OnPropertyChangedEvent(nameof(PageViewModel));
            }
        }

        private bool _isConnected;

        /// <summary>
        /// Uchovává informaci o tom, zda bylo vybráno připojení k jednotce.
        /// </summary>
        public bool IsConnected
        {
            get { return _isConnected; }
            set
            {
                _isConnected = value;
                OnPropertyChangedEvent(nameof(IsConnected));
            }
        }

        private RelayCommand _logout;

        /// <summary>
        /// Příkaz určující, co se má stát při odhlášení.
        /// </summary>
        public ICommand Logout
        {
            get
            {
                return _logout ?? (_logout = new RelayCommand(p => ShowLogin()));
            }
        }

        private RelayCommand _disconnectUnit;

        /// <summary>
        /// Příkaz, který určuje, co se má stát při odpojení od jednotky.
        /// </summary>
        public ICommand DisconnectUnit
        {
            get
            {
                return _disconnectUnit ?? (_disconnectUnit = new RelayCommand(p => ShowConnectionChoice()));
            }
        }

        public MainWindow()
        {
            ShowLogin();
        }

        private void OnLogin(bool successfulLogin)
        {
            if (successfulLogin)
            {
                ShowConnectionChoice();
            }
        }

        private void OnSetConnection(ConnectionType connectionType)
        {
            switch (connectionType)
            {
                case ConnectionType.HID:
                    GlobalInformations.UsingTcp = false;
                    IsConnected = true;
                    OnConnection();
                    break;
                case ConnectionType.TCP:
                    IsConnected = true;
                    var ip = new IpAddressSelection();
                    ip.OnTcpConnection += OnConnection;
                    PageViewModel = ip;
                    break;
            }
        }

        private void OnConnection()
        {
            var mainMenu = new MainMenu();
            mainMenu.OnShowHeatingInformation += OnShowHeatingInformation;
            MenuViewModel = mainMenu;
            PageViewModel = null;
        }

        private void OnShowHeatingInformation()
        {
            var infoRooms = new InfoRoomList();
            infoRooms.OnConnectionError += OnConnectionError;
            infoRooms.RefreshInfoRoom();
            PageViewModel = infoRooms;
        }

        private void OnConnectionError()
        {
            var connectionError = new ConnectionError();
            connectionError.OnConnectionErrorClose += OnConnectionErrorClose;
            FullScreenViewModel = connectionError;
        }

        private void OnConnectionErrorClose()
        {
            FullScreenViewModel = null;
        }

        private void ShowLogin()
        {
            MenuViewModel = null;
            PageViewModel = null;

            var login = new Login();
            login.OnLogin += OnLogin;
            login.OnLoginException += OnLoginException;
            FullScreenViewModel = login;
        }

        private void OnLoginException()
        {
            var loginException = new LoginException();
            loginException.OnLoginExceptionClose += OnLoginExceptionClose;
            FullScreenViewModel = loginException;
        }

        private void OnLoginExceptionClose()
        {
            ShowLogin();
        }

        private void ShowConnectionChoice()
        {
            IsConnected = false;
            FullScreenViewModel = null;
            PageViewModel = null;
            var cs = new ConnectionSelection();
            cs.OnSetConnection += OnSetConnection;
            MenuViewModel = cs;
        }
    }
}
