﻿using System;
using System.Windows.Input;

namespace ViewModel
{
    /// <summary>
    /// Vytvoření třídy pro příkazy - implementace rozhranní ICommand
    /// </summary>
    class RelayCommand : ICommand
    {
        #region Fields

        readonly Action<object> _execute;
        readonly Predicate<object> _canExecute;

        #endregion Fields

        #region Constructors

        public RelayCommand(Action<object> execute)
            : this(execute, null)
        {
        }

        public RelayCommand(Action<object> execute, Predicate<object> canExecute)
        {
            if (execute == null)
                throw new ArgumentNullException("execute");

            _execute = execute;
            _canExecute = canExecute;
        }
        #endregion Constructors

        #region ICommand Members

        /// <summary>
        /// Určuje, zda může být příkaz vykonán, nebo ne.
        /// </summary>
        /// <param name="parameter">parametr</param>
        /// <returns>Zda může být vykonán příkaz</returns>
        public bool CanExecute(object parameter)
        {
            return _canExecute == null ? true : _canExecute(parameter);
        }

        /// <summary>
        /// Událost určující změnu informace, zda může být příkaz vykonán, nebo ne.
        /// </summary>
        public event EventHandler CanExecuteChanged
        {
            add { CommandManager.RequerySuggested += value; }
            remove { CommandManager.RequerySuggested -= value; }
        }

        /// <summary>
        /// Vykonání události.
        /// </summary>
        /// <param name="parameter">parametr</param>
        public void Execute(object parameter)
        {
            _execute(parameter);
        }

        #endregion ICommand Members
    }
}
